@@ -17,7 +17,11 @@ from biosimulators_utils.sedml import validation
 from biosimulators_utils.sedml.data_model import (Task, ModelLanguage, ModelAttributeChange,  # noqa: F401
                                                   UniformTimeCourseSimulation, Variable)
 from biosimulators_utils.sedml.exec import exec_sed_doc
+from biosimulators_utils.simulator.utils import get_algorithm_substitution_policy
 from biosimulators_utils.utils.core import raise_errors_warnings
+from biosimulators_utils.warnings import warn, BioSimulatorsWarning
+from kisao.data_model import AlgorithmSubstitutionPolicy, ALGORITHM_SUBSTITUTION_POLICY_LEVELS
+from kisao.utils import get_preferred_substitute_algorithm_by_ids
 from rpy2.robjects.vectors import StrVector
 import functools
 import numpy
@@ -88,15 +92,15 @@ def exec_sed_task(task, variables, log=None):
                           error_summary='Language for model `{}` is not supported.'.format(model.id))
     raise_errors_warnings(validation.validate_model_change_types(task.model.changes, ()),
                           error_summary='Changes for model `{}` are not supported.'.format(model.id))
-    raise_errors_warnings(validation.validate_model_changes(task.model),
+    raise_errors_warnings(*validation.validate_model_changes(task.model),
                           error_summary='Changes for model `{}` are invalid.'.format(model.id))
     raise_errors_warnings(validation.validate_simulation_type(task.simulation, (UniformTimeCourseSimulation, )),
                           error_summary='{} `{}` is not supported.'.format(sim.__class__.__name__, sim.id))
-    raise_errors_warnings(validation.validate_simulation(task.simulation),
+    raise_errors_warnings(*validation.validate_simulation(task.simulation),
                           error_summary='Simulation `{}` is invalid.'.format(sim.id))
     raise_errors_warnings(validate_time_course(task.simulation),
                           error_summary='Simulation `{}` is invalid.'.format(sim.id))
-    raise_errors_warnings(validation.validate_data_generator_variables(variables),
+    raise_errors_warnings(*validation.validate_data_generator_variables(variables),
                           error_summary='Data generator variables for task `{}` are invalid.'.format(task.id))
     target_x_paths_keys = get_variable_target_x_path_keys(variables, task.model.source)
 
@@ -121,23 +125,41 @@ def exec_sed_task(task, variables, log=None):
 
     # Load the algorithm specified by :obj:`task.simulation.algorithm.kisao_id`
     alg_kisao_id = sim.algorithm.kisao_id
-    alg = KISAO_METHOD_ARGUMENTS_MAP.get(alg_kisao_id, None)
-    if alg is None:
-        raise NotImplementedError("".join([
-            "Algorithm with KiSAO id '{}' is not supported. ".format(alg_kisao_id),
-            "Algorithm must have one of the following KiSAO ids:\n  - {}".format('\n  - '.join(
-                '{}: {}'.format(kisao_id, alg['name'])
-                for kisao_id, alg in KISAO_METHOD_ARGUMENTS_MAP.items())),
-        ]))
-
+    algorithm_substitution_policy = get_algorithm_substitution_policy()
+    exec_kisao_id = get_preferred_substitute_algorithm_by_ids(
+        alg_kisao_id, KISAO_METHOD_ARGUMENTS_MAP.keys(),
+        substitution_policy=algorithm_substitution_policy)
+    alg = KISAO_METHOD_ARGUMENTS_MAP[exec_kisao_id]
     simulation_method_args['type'] = StrVector([alg['type']])
 
     # Apply the algorithm parameter changes specified by `simulation.algorithm.parameter_changes`
-    for change in sim.algorithm.changes:
-        set_simulation_method_arg(model, alg_kisao_id, change, simulation_method_args)
+    if exec_kisao_id == alg_kisao_id:
+        for change in sim.algorithm.changes:
+            try:
+                set_simulation_method_arg(model, exec_kisao_id, change, simulation_method_args)
+            except NotImplementedError as exception:
+                if (
+                    ALGORITHM_SUBSTITUTION_POLICY_LEVELS[algorithm_substitution_policy]
+                    > ALGORITHM_SUBSTITUTION_POLICY_LEVELS[AlgorithmSubstitutionPolicy.NONE]
+                ):
+                    warn('Unsuported algorithm parameter `{}` was ignored:\n  {}'.format(
+                        change.kisao_id, str(exception).replace('\n', '\n  ')),
+                        BioSimulatorsWarning)
+                else:
+                    raise
+            except ValueError as exception:
+                if (
+                    ALGORITHM_SUBSTITUTION_POLICY_LEVELS[algorithm_substitution_policy]
+                    > ALGORITHM_SUBSTITUTION_POLICY_LEVELS[AlgorithmSubstitutionPolicy.NONE]
+                ):
+                    warn('Unsuported value `{}` for algorithm parameter `{}` was ignored:\n  {}'.format(
+                        change.new_value, change.kisao_id, str(exception).replace('\n', '\n  ')),
+                        BioSimulatorsWarning)
+                else:
+                    raise
 
     # validate that BoolNet can produce the desired variables of the desired data generators
-    validate_data_generator_variables(variables, alg_kisao_id)
+    validate_data_generator_variables(variables, exec_kisao_id)
 
     # execute simulation
     species_results_matrix = boolnet.generateTimeSeries(model, **simulation_method_args)[0]
@@ -151,7 +173,7 @@ def exec_sed_task(task, variables, log=None):
         variable_results[variable.id] = variable_results[variable.id][-(int(sim.number_of_points) + 1):]
 
     # log action
-    log.algorithm = alg_kisao_id
+    log.algorithm = exec_kisao_id
     log.simulator_details = {
         'method': 'BoolNet::generateTimeSeries',
         'arguments': simulation_method_args,
